<?php

namespace Drupal\Tests\field_encrypt\Functional;

use Drupal\field_encrypt\ProcessEntities;
use Drupal\Tests\encrypt\Functional\EncryptTestBase;
use Drupal\Tests\node\Traits\NodeCreationTrait;
use Drupal\Tests\Traits\Core\CronRunTrait;

/**
 * Tests Field Encrypt's API hooks.
 *
 * @group field_encrypt
 * @see hook_field_encrypt_unencrypted_storage_value_alter()
 * @see hook_field_encrypt_allow_encryption()
 */
class FieldEncryptApiHooksTest extends EncryptTestBase {

  use CronRunTrait;
  use NodeCreationTrait;

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'node',
    'field',
    'text',
    'key',
    'encrypt',
    'encrypt_test',
    'field_encrypt',
    'field_encrypt_test',
  ];

  /**
   * {@inheritdoc}
   *
   * @TODO: Simplify setUp() by extending EncryptTestBase when https://www.drupal.org/node/2692387 lands.
   */
  protected function setUp() {
    parent::setUp();

    // Create an admin user.
    $this->adminUser = $this->drupalCreateUser([
      'access administration pages',
      'administer encrypt',
      'administer keys',
      'administer field encryption',
    ], NULL, TRUE);
    $this->drupalLogin($this->adminUser);

    // Create content type to test.
    $this->drupalCreateContentType(['type' => 'page', 'name' => 'Basic page']);

    $this->config('field_encrypt.settings')
      ->set('encryption_profile', 'encryption_profile_1')
      ->save();
  }

  /**
   * Set up base fields for test.
   *
   * @param bool $encryption
   *   Whether or not the fields should be encrypted. Defaults to TRUE.
   */
  protected function setFieldStorageSettings($encryption = TRUE) {
    // Set up storage settings for first field.
    $this->drupalGet('admin/config/system/field-encrypt/entity-types');
    $this->assertSession()->fieldExists('entity_type')->selectOption('Content');
    $this->submitForm([], 'Save configuration');
    if ($encryption) {
      $this->assertSession()->fieldExists('base_fields[title]')->check();
    }
    else {
      $this->assertSession()->fieldExists('base_fields[title]')->uncheck();
    }
    $this->submitForm([], 'Save configuration');
    $this->assertSession()->pageTextContains('Updated encryption settings for Content base fields.');
    $this->rebuildAll();
  }

  /**
   * Tests field_encrypt hooks.
   */
  public function testHooks() {
    $this->setFieldStorageSettings(TRUE);
    \Drupal::state()->set('field_encrypt_test.hook_field_encrypt_unencrypted_storage_value_alter', TRUE);
    // Save test entity.
    $this->drupalGet('node/add/page');
    $this->assertSession()->fieldExists('title[0][value]')->setValue('Test title');
    $this->assertSession()->buttonExists('Save')->press();

    // Check the node title is displayed unencrypted.
    $this->assertSession()->elementTextContains('css', 'h1', "Test title");
    // Assert the arguments passed to hook are as expected.
    $this->assertSession()->pageTextContains("Value alter hook: Entity title: Test title");
    $this->assertSession()->pageTextContains("Value alter hook: Field name: title");
    $this->assertSession()->pageTextContains("Value alter hook: Property: value");

    $result = \Drupal::database()->query("SELECT title FROM {node_field_data} WHERE nid = 1")->fetchField();
    $this->assertEquals('🐒', $result);

    \Drupal::state()->set('field_encrypt_test.hook_field_encrypt_unencrypted_storage_value_alter', FALSE);
    \Drupal::state()->set('field_encrypt_test.hook_field_encrypt_allow_encryption', TRUE);
    $this->drupalGet('node/1/edit');
    $this->assertSession()->buttonExists('Save')->press();
    $result = \Drupal::database()->query("SELECT title FROM {node_field_data} WHERE nid = 1")->fetchField();
    $this->assertEquals('Test title', $result);
    // Check the node title is displayed unencrypted.
    $this->assertSession()->elementTextContains('css', 'h1', "Test title");
    // Assert the arguments passed to hook are as expected.
    $this->assertSession()->pageTextContains("Allow encryption hook: Entity title: Test title");

    $this->drupalGet('node/1/edit');
    $this->assertSession()->fieldExists('status[value]')->uncheck();
    $this->assertSession()->buttonExists('Save')->press();
    $result = \Drupal::database()->query("SELECT title FROM {node_field_data} WHERE nid = 1")->fetchField();
    $this->assertEquals(ProcessEntities::ENCRYPTED_VALUE, $result);
    // Check the node title is displayed unencrypted.
    $this->assertSession()->elementTextContains('css', 'h1', "Test title");
    // Assert the arguments passed to hook are as expected.
    $this->assertSession()->pageTextContains("Allow encryption hook: Entity title: Test title");
  }

}
